@@ -22,58 +22,60 @@ import org.apache.flink.api.common.typeinfo.TypeInformation;
 import org.apache.flink.api.common.typeinfo.Types;
 import org.apache.flink.api.java.typeutils.RowTypeInfo;
 import org.apache.flink.configuration.Configuration;
+import org.apache.flink.connector.hbase.sink.HBaseSinkFunction;
+import org.apache.flink.connector.hbase.sink.RowDataToMutationConverter;
+import org.apache.flink.connector.hbase.util.HBaseConfigurationUtil;
 import org.apache.flink.connector.hbase.util.HBaseTableSchema;
 import org.apache.flink.connector.hbase2.source.AbstractTableInputFormat;
 import org.apache.flink.connector.hbase2.source.HBaseRowDataInputFormat;
 import org.apache.flink.connector.hbase2.util.HBaseTestBase;
-import org.apache.flink.runtime.testutils.MiniClusterResourceConfiguration;
 import org.apache.flink.streaming.api.datastream.DataStream;
 import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
 import org.apache.flink.table.api.Table;
 import org.apache.flink.table.api.TableEnvironment;
 import org.apache.flink.table.api.TableResult;
 import org.apache.flink.table.api.bridge.java.StreamTableEnvironment;
+import org.apache.flink.table.data.RowData;
 import org.apache.flink.table.functions.ScalarFunction;
-import org.apache.flink.test.util.MiniClusterWithClientResource;
+import org.apache.flink.table.planner.factories.TestValuesTableFactory;
 import org.apache.flink.test.util.TestBaseUtils;
 import org.apache.flink.types.Row;
 import org.apache.flink.types.RowKind;
 import org.apache.flink.util.CollectionUtil;
 
-import org.apache.flink.shaded.guava30.com.google.common.collect.Lists;
-
+import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.TableName;
+import org.apache.hadoop.hbase.TableNotFoundException;
 import org.apache.hadoop.hbase.util.Bytes;
-import org.junit.ClassRule;
-import org.junit.Test;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.EnumSource;
 
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Spliterator;
+import java.util.Spliterators;
+import java.util.concurrent.TimeUnit;
 import java.util.stream.Collectors;
+import java.util.stream.StreamSupport;
 
 import static org.apache.flink.table.api.Expressions.$;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertNull;
+import static org.assertj.core.api.Assertions.assertThat;
+import static org.assertj.core.api.Assertions.assertThatThrownBy;
 
 /** IT cases for HBase connector (including source and sink). */
-public class HBaseConnectorITCase extends HBaseTestBase {
-
-    @ClassRule
-    public static final MiniClusterWithClientResource MINI_CLUSTER =
-            new MiniClusterWithClientResource(
-                    new MiniClusterResourceConfiguration.Builder()
-                            .setConfiguration(new Configuration())
-                            .build());
+class HBaseConnectorITCase extends HBaseTestBase {
 
     // -------------------------------------------------------------------------------------
     // HBaseTableSource tests
     // -------------------------------------------------------------------------------------
 
     @Test
-    public void testTableSourceFullScan() {
+    void testTableSourceFullScan() {
         TableEnvironment tEnv = TableEnvironment.create(batchSettings);
 
         tEnv.executeSql(
@@ -119,7 +121,32 @@ public class HBaseConnectorITCase extends HBaseTestBase {
     }
 
     @Test
-    public void testTableSourceProjection() {
+    void testTableSourceEmptyTableScan() {
+        TableEnvironment tEnv = TableEnvironment.create(batchSettings);
+
+        tEnv.executeSql(
+                "CREATE TABLE hTable ("
+                        + " family1 ROW<col1 INT>,"
+                        + " rowkey INT,"
+                        + " PRIMARY KEY (rowkey) NOT ENFORCED"
+                        + ") WITH ("
+                        + " 'connector' = 'hbase-2.2',"
+                        + " 'table-name' = '"
+                        + TEST_EMPTY_TABLE
+                        + "',"
+                        + " 'zookeeper.quorum' = '"
+                        + getZookeeperQuorum()
+                        + "'"
+                        + ")");
+
+        Table table = tEnv.sqlQuery("SELECT rowkey, h.family1.col1 FROM hTable AS h");
+        List<Row> results = CollectionUtil.iteratorToList(table.execute().collect());
+
+        assertThat(results).isEmpty();
+    }
+
+    @Test
+    void testTableSourceProjection() {
         TableEnvironment tEnv = TableEnvironment.create(batchSettings);
 
         tEnv.executeSql(
@@ -163,7 +190,7 @@ public class HBaseConnectorITCase extends HBaseTestBase {
     }
 
     @Test
-    public void testTableSourceFieldOrder() {
+    void testTableSourceFieldOrder() {
         TableEnvironment tEnv = TableEnvironment.create(batchSettings);
 
         tEnv.executeSql(
@@ -199,7 +226,7 @@ public class HBaseConnectorITCase extends HBaseTestBase {
     }
 
     @Test
-    public void testTableSourceReadAsByteArray() {
+    void testTableSourceReadAsByteArray() {
         TableEnvironment tEnv = TableEnvironment.create(batchSettings);
 
         tEnv.executeSql(
@@ -241,7 +268,7 @@ public class HBaseConnectorITCase extends HBaseTestBase {
     }
 
     @Test
-    public void testTableSink() throws Exception {
+    void testTableSink() throws Exception {
         StreamExecutionEnvironment execEnv = StreamExecutionEnvironment.getExecutionEnvironment();
         StreamTableEnvironment tEnv = StreamTableEnvironment.create(execEnv, streamSettings);
 
@@ -268,8 +295,7 @@ public class HBaseConnectorITCase extends HBaseTestBase {
         // wait to finish
         tableResult.await();
 
-        assertEquals(
-                "Expected INSERT rowKind", RowKind.INSERT, tableResult.collect().next().getKind());
+        assertThat(tableResult.collect().next().getKind()).isEqualTo(RowKind.INSERT);
 
         // start a batch scan job to verify contents in HBase table
         TableEnvironment batchEnv = TableEnvironment.create(batchSettings);
@@ -288,7 +314,8 @@ public class HBaseConnectorITCase extends HBaseTestBase {
         Table countTable =
                 batchEnv.sqlQuery("SELECT COUNT(h.rowkey) FROM " + TEST_TABLE_2 + " AS h");
 
-        assertEquals(new Long(expected.size()), countTable.execute().collect().next().getField(0));
+        assertThat(countTable.execute().collect().next().getField(0))
+                .isEqualTo(new Long(expected.size()));
 
         Table table =
                 batchEnv.sqlQuery(
@@ -312,7 +339,178 @@ public class HBaseConnectorITCase extends HBaseTestBase {
     }
 
     @Test
-    public void testTableSourceSinkWithDDL() throws Exception {
+    void testTableSinkWithChangelog() throws Exception {
+        StreamExecutionEnvironment execEnv = StreamExecutionEnvironment.getExecutionEnvironment();
+        StreamTableEnvironment tEnv = StreamTableEnvironment.create(execEnv, streamSettings);
+
+        // register values table for source
+        String dataId =
+                TestValuesTableFactory.registerData(
+                        Arrays.asList(
+                                Row.ofKind(RowKind.INSERT, 1, Row.of("Hello1")),
+                                Row.ofKind(RowKind.DELETE, 1, Row.of("Hello2")),
+                                Row.ofKind(RowKind.INSERT, 2, Row.of("Hello1")),
+                                Row.ofKind(RowKind.INSERT, 2, Row.of("Hello2")),
+                                Row.ofKind(RowKind.INSERT, 2, Row.of("Hello3")),
+                                Row.ofKind(RowKind.DELETE, 2, Row.of("Hello3")),
+                                Row.ofKind(RowKind.INSERT, 1, Row.of("Hello3"))));
+        tEnv.executeSql(
+                "CREATE TABLE source_table ("
+                        + " rowkey INT,"
+                        + " family1 ROW<name STRING>,"
+                        + " PRIMARY KEY (rowkey) NOT ENFORCED"
+                        + ") WITH ("
+                        + " 'connector' = 'values',"
+                        + " 'data-id' = '"
+                        + dataId
+                        + "',"
+                        + " 'changelog-mode'='I,UA,UB,D'"
+                        + ")");
+
+        // register HBase table for sink
+        tEnv.executeSql(
+                "CREATE TABLE sink_table ("
+                        + " rowkey INT,"
+                        + " family1 ROW<name STRING>,"
+                        + " PRIMARY KEY (rowkey) NOT ENFORCED"
+                        + ") WITH ("
+                        + " 'connector' = 'hbase-2.2',"
+                        + " 'table-name' = '"
+                        + TEST_TABLE_4
+                        + "',"
+                        + " 'zookeeper.quorum' = '"
+                        + getZookeeperQuorum()
+                        + "'"
+                        + ")");
+
+        tEnv.executeSql("INSERT INTO sink_table SELECT * FROM source_table").await();
+
+        TableResult result = tEnv.executeSql("SELECT * FROM sink_table");
+
+        List<Row> actual = CollectionUtil.iteratorToList(result.collect());
+        assertThat(actual).isEqualTo(Collections.singletonList(Row.of(1, Row.of("Hello3"))));
+    }
+
+    @Test
+    void testTableSinkWithTimestampMetadata() throws Exception {
+        StreamExecutionEnvironment execEnv = StreamExecutionEnvironment.getExecutionEnvironment();
+        StreamTableEnvironment tEnv = StreamTableEnvironment.create(execEnv, streamSettings);
+
+        tEnv.executeSql(
+                "CREATE TABLE hTableForSink ("
+                        + " rowkey INT PRIMARY KEY NOT ENFORCED,"
+                        + " family1 ROW<col1 INT>,"
+                        + " version TIMESTAMP_LTZ(3) NOT NULL METADATA FROM 'timestamp'"
+                        + ") WITH ("
+                        + " 'connector' = 'hbase-2.2',"
+                        + " 'table-name' = '"
+                        + TEST_TABLE_5
+                        + "',"
+                        + " 'zookeeper.quorum' = '"
+                        + getZookeeperQuorum()
+                        + "'"
+                        + ")");
+
+        String insert =
+                "INSERT INTO hTableForSink VALUES"
+                        + "(1, ROW(1), TO_TIMESTAMP_LTZ(1696767943270, 3)),"
+                        + "(2, ROW(2), TO_TIMESTAMP_LTZ(1696767943270, 3)),"
+                        + "(3, ROW(3), TO_TIMESTAMP_LTZ(1696767943270, 3)),"
+                        + "(1, ROW(10), TO_TIMESTAMP_LTZ(1696767943269, 3)),"
+                        + "(2, ROW(20), TO_TIMESTAMP_LTZ(1696767943271, 3))";
+        tEnv.executeSql(insert).await();
+
+        tEnv.executeSql(
+                "CREATE TABLE hTableForQuery ("
+                        + " rowkey INT PRIMARY KEY NOT ENFORCED,"
+                        + " family1 ROW<col1 INT>"
+                        + ") WITH ("
+                        + " 'connector' = 'hbase-2.2',"
+                        + " 'table-name' = '"
+                        + TEST_TABLE_5
+                        + "',"
+                        + " 'zookeeper.quorum' = '"
+                        + getZookeeperQuorum()
+                        + "'"
+                        + ")");
+        TableResult result = tEnv.executeSql("SELECT rowkey, family1.col1 FROM hTableForQuery");
+        List<Row> results = CollectionUtil.iteratorToList(result.collect());
+
+        String expected = "+I[1, 1]\n+I[2, 20]\n+I[3, 3]\n";
+
+        TestBaseUtils.compareResultAsText(results, expected);
+    }
+
+    @Test
+    void testTableSinkWithTTLMetadata() throws Exception {
+        StreamExecutionEnvironment execEnv = StreamExecutionEnvironment.getExecutionEnvironment();
+        StreamTableEnvironment tEnv = StreamTableEnvironment.create(execEnv, streamSettings);
+
+        tEnv.executeSql(
+                "CREATE TABLE hTableForSink ("
+                        + " rowkey INT PRIMARY KEY NOT ENFORCED,"
+                        + " family1 ROW<col1 INT>,"
+                        + " ttl BIGINT NOT NULL METADATA FROM 'ttl'"
+                        + ") WITH ("
+                        + " 'connector' = 'hbase-2.2',"
+                        + " 'table-name' = '"
+                        + TEST_TABLE_6
+                        + "',"
+                        + " 'zookeeper.quorum' = '"
+                        + getZookeeperQuorum()
+                        + "'"
+                        + ")");
+
+        String insert =
+                "INSERT INTO hTableForSink VALUES"
+                        + "(1, ROW(1), 2000),"
+                        + "(2, ROW(2), 9000),"
+                        + "(3, ROW(3), 5000)";
+        tEnv.executeSql(insert).await();
+
+        tEnv.executeSql(
+                "CREATE TABLE hTableForQuery ("
+                        + " rowkey INT PRIMARY KEY NOT ENFORCED,"
+                        + " family1 ROW<col1 INT>"
+                        + ") WITH ("
+                        + " 'connector' = 'hbase-2.2',"
+                        + " 'table-name' = '"
+                        + TEST_TABLE_6
+                        + "',"
+                        + " 'zookeeper.quorum' = '"
+                        + getZookeeperQuorum()
+                        + "'"
+                        + ")");
+        String query = "SELECT rowkey, family1.col1 FROM hTableForQuery";
+
+        TableResult firstResult = tEnv.executeSql(query);
+        List<Row> firstResults = CollectionUtil.iteratorToList(firstResult.collect());
+        String firstExpected = "+I[1, 1]\n+I[2, 2]\n+I[3, 3]\n";
+        TestBaseUtils.compareResultAsText(firstResults, firstExpected);
+
+        TimeUnit.SECONDS.sleep(3);
+
+        TableResult secondResult = tEnv.executeSql(query);
+        List<Row> secondResults = CollectionUtil.iteratorToList(secondResult.collect());
+        String secondExpected = "+I[2, 2]\n+I[3, 3]\n";
+        TestBaseUtils.compareResultAsText(secondResults, secondExpected);
+
+        TimeUnit.SECONDS.sleep(3);
+
+        TableResult thirdResult = tEnv.executeSql(query);
+        List<Row> thirdResults = CollectionUtil.iteratorToList(thirdResult.collect());
+        String thirdExpected = "+I[2, 2]";
+        TestBaseUtils.compareResultAsText(thirdResults, thirdExpected);
+
+        TimeUnit.SECONDS.sleep(4);
+
+        TableResult lastResult = tEnv.executeSql(query);
+        List<Row> lastResults = CollectionUtil.iteratorToList(lastResult.collect());
+        assertThat(lastResults).isEmpty();
+    }
+
+    @Test
+    void testTableSourceSinkWithDDL() throws Exception {
         StreamExecutionEnvironment execEnv = StreamExecutionEnvironment.getExecutionEnvironment();
         StreamTableEnvironment tEnv = StreamTableEnvironment.create(execEnv, streamSettings);
 
@@ -340,8 +538,7 @@ public class HBaseConnectorITCase extends HBaseTestBase {
         // wait to finish
         tableResult.await();
 
-        assertEquals(
-                "Expected INSERT rowKind", RowKind.INSERT, tableResult.collect().next().getKind());
+        assertThat(tableResult.collect().next().getKind()).isEqualTo(RowKind.INSERT);
 
         // start a batch scan job to verify contents in HBase table
         TableEnvironment batchEnv = TableEnvironment.create(batchSettings);
@@ -385,42 +582,165 @@ public class HBaseConnectorITCase extends HBaseTestBase {
         TableResult tableResult3 = batchEnv.executeSql(query);
 
         List<String> result =
-                Lists.newArrayList(tableResult3.collect()).stream()
+                StreamSupport.stream(
+                                Spliterators.spliteratorUnknownSize(
+                                        tableResult3.collect(), Spliterator.ORDERED),
+                                false)
                         .map(Row::toString)
                         .sorted()
                         .collect(Collectors.toList());
 
-        assertEquals(expected, result);
+        assertThat(result).isEqualTo(expected);
+    }
+
+    @ParameterizedTest
+    @EnumSource(Caching.class)
+    void testHBaseLookupTableSource(Caching caching) {
+        verifyHBaseLookupJoin(caching, false);
+    }
+
+    @ParameterizedTest
+    @EnumSource(Caching.class)
+    void testHBaseAsyncLookupTableSource(Caching caching) {
+        verifyHBaseLookupJoin(caching, true);
     }
 
     @Test
-    public void testHBaseLookupTableSource() {
-        verifyHBaseLookupJoin(false);
-    }
-
-    @Test
-    public void testHBaseAsyncLookupTableSource() {
-        verifyHBaseLookupJoin(true);
-    }
-
-    @Test
-    public void testTableInputFormatOpenClose() throws IOException {
+    void testTableInputFormatOpenClose() throws IOException {
         HBaseTableSchema tableSchema = new HBaseTableSchema();
         tableSchema.addColumn(FAMILY1, F1COL1, byte[].class);
         AbstractTableInputFormat<?> inputFormat =
                 new HBaseRowDataInputFormat(getConf(), TEST_TABLE_1, tableSchema, "null");
         inputFormat.open(inputFormat.createInputSplits(1)[0]);
-        assertNotNull(inputFormat.getConnection());
-        assertNotNull(inputFormat.getConnection().getTable(TableName.valueOf(TEST_TABLE_1)));
+        assertThat(inputFormat.getConnection()).isNotNull();
+        assertThat(inputFormat.getConnection().getTable(TableName.valueOf(TEST_TABLE_1)))
+                .isNotNull();
 
         inputFormat.close();
-        assertNull(inputFormat.getConnection());
+        assertThat(inputFormat.getConnection()).isNull();
     }
 
-    private void verifyHBaseLookupJoin(boolean async) {
+    @Test
+    void testTableInputFormatTableExistence() throws IOException {
+        HBaseTableSchema tableSchema = new HBaseTableSchema();
+        tableSchema.addColumn(FAMILY1, F1COL1, byte[].class);
+        AbstractTableInputFormat<?> inputFormat =
+                new HBaseRowDataInputFormat(getConf(), TEST_NOT_EXISTS_TABLE, tableSchema, "null");
+
+        assertThatThrownBy(() -> inputFormat.createInputSplits(1))
+                .isExactlyInstanceOf(TableNotFoundException.class);
+
+        inputFormat.close();
+        assertThat(inputFormat.getConnection()).isNull();
+    }
+
+    @Test
+    void testHBaseSinkFunctionTableExistence() throws Exception {
+        org.apache.hadoop.conf.Configuration hbaseConf =
+                HBaseConfigurationUtil.getHBaseConfiguration();
+        hbaseConf.set(HConstants.ZOOKEEPER_QUORUM, getZookeeperQuorum());
+        hbaseConf.set(HConstants.ZOOKEEPER_ZNODE_PARENT, "/hbase");
+
+        HBaseTableSchema tableSchema = new HBaseTableSchema();
+        tableSchema.addColumn(FAMILY1, F1COL1, byte[].class);
+
+        HBaseSinkFunction<RowData> sinkFunction =
+                new HBaseSinkFunction<>(
+                        TEST_NOT_EXISTS_TABLE,
+                        hbaseConf,
+                        new RowDataToMutationConverter(
+                                tableSchema,
+                                tableSchema.convertToDataType(),
+                                Collections.emptyList(),
+                                "null",
+                                false),
+                        2 * 1024 * 1024,
+                        1000,
+                        1000);
+
+        assertThatThrownBy(() -> sinkFunction.open(new Configuration()))
+                .getRootCause()
+                .isExactlyInstanceOf(TableNotFoundException.class);
+
+        sinkFunction.close();
+    }
+
+    @Test
+    void testTableSinkDisabledBufferFlush() throws Exception {
         StreamExecutionEnvironment execEnv = StreamExecutionEnvironment.getExecutionEnvironment();
         StreamTableEnvironment tEnv = StreamTableEnvironment.create(execEnv, streamSettings);
 
+        tEnv.executeSql(
+                "CREATE TABLE hTableForSink ("
+                        + " rowkey INT PRIMARY KEY NOT ENFORCED,"
+                        + " family1 ROW<col1 INT>"
+                        + ") WITH ("
+                        + " 'connector' = 'hbase-2.2',"
+                        + " 'sink.buffer-flush.max-size' = '0',"
+                        + " 'sink.buffer-flush.max-rows' = '0',"
+                        + " 'table-name' = '"
+                        + TEST_TABLE_7
+                        + "',"
+                        + " 'zookeeper.quorum' = '"
+                        + getZookeeperQuorum()
+                        + "'"
+                        + ")");
+
+        String insert = "INSERT INTO hTableForSink VALUES(1, ROW(1))";
+        tEnv.executeSql(insert).await();
+
+        tEnv.executeSql(
+                "CREATE VIEW user_click AS "
+                        + " SELECT user_id, proctime() AS proc_time"
+                        + " FROM ( "
+                        + " VALUES(1), (1), (1), (1), (1)"
+                        + " ) AS t (user_id);");
+
+        tEnv.executeSql(
+                "INSERT INTO hTableForSink SELECT "
+                        + "    user_id as rowkey,"
+                        + "    ROW(CAST(family1.col1 + 1 AS INT))"
+                        + " FROM user_click INNER JOIN hTableForSink"
+                        + " FOR SYSTEM_TIME AS OF user_click.proc_time"
+                        + " ON hTableForSink.rowkey = user_click.user_id;");
+
+        tEnv.executeSql(
+                "CREATE TABLE hTableForQuery ("
+                        + " rowkey INT PRIMARY KEY NOT ENFORCED,"
+                        + " family1 ROW<col1 INT>"
+                        + ") WITH ("
+                        + " 'connector' = 'hbase-2.2',"
+                        + " 'table-name' = '"
+                        + TEST_TABLE_7
+                        + "',"
+                        + " 'zookeeper.quorum' = '"
+                        + getZookeeperQuorum()
+                        + "'"
+                        + ")");
+        String query = "SELECT rowkey, family1.col1 FROM hTableForQuery";
+
+        TableResult firstResult = tEnv.executeSql(query);
+        List<Row> firstResults = CollectionUtil.iteratorToList(firstResult.collect());
+        String firstExpected = "+I[1, 6]";
+        TestBaseUtils.compareResultAsText(firstResults, firstExpected);
+    }
+
+    private void verifyHBaseLookupJoin(Caching caching, boolean async) {
+        StreamExecutionEnvironment execEnv = StreamExecutionEnvironment.getExecutionEnvironment();
+        StreamTableEnvironment tEnv = StreamTableEnvironment.create(execEnv, streamSettings);
+
+        String cacheOptions = "";
+        if (caching == Caching.ENABLE_CACHE) {
+            cacheOptions =
+                    ","
+                            + String.join(
+                                    ",",
+                                    Arrays.asList(
+                                            "'lookup.cache' = 'PARTIAL'",
+                                            "'lookup.partial-cache.max-rows' = '1000'",
+                                            "'lookup.partial-cache.expire-after-write' = '10min'"));
+        }
+
         tEnv.executeSql(
                 "CREATE TABLE "
                         + TEST_TABLE_1
@@ -442,6 +762,7 @@ public class HBaseConnectorITCase extends HBaseTestBase {
                         + " 'zookeeper.quorum' = '"
                         + getZookeeperQuorum()
                         + "'"
+                        + cacheOptions
                         + ")");
 
         // prepare a source table
@@ -470,12 +791,16 @@ public class HBaseConnectorITCase extends HBaseTestBase {
                         + " FOR SYSTEM_TIME AS OF src.proc as h ON src.a = h.rowkey";
         Iterator<Row> collected = tEnv.executeSql(dimJoinQuery).collect();
         List<String> result =
-                Lists.newArrayList(collected).stream()
+                StreamSupport.stream(
+                                Spliterators.spliteratorUnknownSize(collected, Spliterator.ORDERED),
+                                false)
                         .map(Row::toString)
                         .sorted()
                         .collect(Collectors.toList());
 
         List<String> expected = new ArrayList<>();
+        expected.add(
+                "+I[1, 1, 10, Hello-1, 100, 1.01, false, Welt-1, 2019-08-18T19:00, 2019-08-18, 19:00, 12345678.0001]");
         expected.add(
                 "+I[1, 1, 10, Hello-1, 100, 1.01, false, Welt-1, 2019-08-18T19:00, 2019-08-18, 19:00, 12345678.0001]");
         expected.add(
@@ -485,7 +810,7 @@ public class HBaseConnectorITCase extends HBaseTestBase {
         expected.add(
                 "+I[3, 3, 30, Hello-3, 300, 3.03, false, Welt-3, 2019-08-18T19:02, 2019-08-18, 19:02, 12345678.0003]");
 
-        assertEquals(expected, result);
+        assertThat(result).isEqualTo(expected);
     }
 
     // -------------------------------------------------------------------------------------
@@ -504,6 +829,12 @@ public class HBaseConnectorITCase extends HBaseTestBase {
         testData.add(Row.of(2, 2L, "Hello"));
         testData.add(Row.of(3, 2L, "Hello world"));
         testData.add(Row.of(3, 3L, "Hello world!"));
+        testData.add(Row.of(1, 1L, "Hi")); // lookup one more time
+    }
+
+    private enum Caching {
+        ENABLE_CACHE,
+        DISABLE_CACHE
     }
 
     // ------------------------------- Utilities -------------------------------------------------
